﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Threading.Tasks;
using Microsoft.ApplicationInsights;
using Microsoft.ApplicationInsights.DataContracts;
using Microsoft.AspNetCore.Mvc;
using NTK2019.Web.Extensions;
using NTK2019.Web.Models;

namespace NTK2019.Web.Controllers
{
    public class TrackController : Controller
    {
        private readonly TelemetryClient telemetry;

        public TrackController(TelemetryClient telemetry)
        {
            this.telemetry = telemetry;
            telemetry.Context.GlobalProperties["Controller"] = "TrackController";
        }

        public IActionResult Index()
        {
            telemetry.TrackPageView("TrackEventHome");
            telemetry.Flush();
            return View();
        }

        public IActionResult TrackPageViews()
        {
            telemetry.TrackPageView("TrackPageViews");
            telemetry.Flush();
            return View();
        }

        public IActionResult TrackDependency()
        {
            telemetry.TrackPageView("TrackDependency");
            telemetry.Flush();
            return View(new TrackDependencyViewModel());
        }

        [HttpPost]
        public async Task<IActionResult> TrackDependency(TrackDependencyViewModel trackDependencyViewModel)
        {
            telemetry.TrackPageView("TrackDependencySubmitted");

            var success = false;
            var startTime = DateTime.UtcNow;
            var timer = System.Diagnostics.Stopwatch.StartNew();
            try
            {
                await Task.Delay(trackDependencyViewModel.Seconds / 1000);
                success = true;
            }
            finally
            {
                timer.Stop();
                telemetry.TrackDependency("NTK2019.Web.Controllers.TrackDependency", "thirdpartylibrary",
                    $"Poslano sporočilo ob {DateTime.Now.ToString(CultureInfo.InvariantCulture)}", startTime, timer.Elapsed,
                    success);
            }

            return RedirectToAction("TrackDependency");
        }

        public IActionResult TrackTrace()
        {
            telemetry.TrackPageView("TrackTrace");
            telemetry.Flush();
            return View(new TraceViewModel());
        }

        [HttpPost]
        public IActionResult TrackTrace(TraceViewModel traceViewModel)
        {
            telemetry.TrackPageView("TrackTraceSubmitted");
            var severityLevel = traceViewModel.SeverityLevel.ParseAsEnum<SeverityLevel>();
            telemetry.TrackTrace(traceViewModel.Message, severityLevel);
            TempData["MessageTrackTrace"] =
                $"Poslano sporočilo {traceViewModel.Message} z levelom {traceViewModel.SeverityLevel}";
            telemetry.Flush();
            return RedirectToAction("TrackTrace");
        }

        public IActionResult TrackException()
        {
            telemetry.TrackPageView("TrackException");
            telemetry.Flush();
            return View(new ExceptionViewModel());
        }

        [HttpPost]
        public IActionResult TrackException(ExceptionViewModel exceptionViewModel)
        {
            telemetry.TrackEvent("TrackExceptionSubmitted");

            try
            {
                throw new Exception(exceptionViewModel.Value);
            }
            catch (Exception e)
            {
                telemetry.TrackException(e, new Dictionary<string, string>
                {
                    {
                        "messageFromUser",
                        $"{exceptionViewModel.Value} at {DateTime.Now.ToShortDateString()}"
                    }
                });
            }
            finally { telemetry.Flush(); }

            TempData["MessageTrackException"] =
                $"Poslana napaka {exceptionViewModel.Value} v nadaljno obdelavo.";

            return RedirectToAction("TrackException");
        }

        public IActionResult TrackEvent()
        {
            telemetry.TrackEvent("TrackEvent");
            telemetry.Flush();
            return View(new TrackEventViewModel());
        }

        [HttpPost]
        public IActionResult TrackEvent(TrackEventViewModel trackEventViewModel)
        {
            telemetry.TrackEvent("TrackEventSubmitted");

            var trackCount = trackEventViewModel.Count;
            for (int currentCount = 0; currentCount < trackCount; currentCount++)
            {
                telemetry.TrackEvent(trackEventViewModel.Name, new Dictionary<string, string>
                {
                    {
                        $"trackEventViewModel.Name{currentCount}", trackEventViewModel.Value
                    }
                });
            }

            TempData["MessageTrackEvent"] =
                $"Poslano število {trackEventViewModel.Count} dogodkov (z imenom {trackEventViewModel.Name}) v obdelavo.";

            telemetry.Flush();

            return RedirectToAction("TrackEvent");
        }

        [HttpPost]
        public IActionResult TrackEventAggregation(TrackEventViewModel trackEventViewModel)
        {
            telemetry.TrackEvent("TrackEventAggregation");

            var trackCount = trackEventViewModel.Count;
            for (int currentCount = 0; currentCount < trackCount; currentCount++)
            {
                telemetry.GetMetric("TrackEventAggregationValues").TrackValue(double.Parse(trackEventViewModel.Value));
            }

            TempData["MessageTrackEvent"] =
                $"Poslano {trackEventViewModel.Count} sporočil v enem klicu {trackEventViewModel.Name} v nadaljno obdelavo.";

            telemetry.Flush();

            return RedirectToAction("TrackEvent");
        }
    }
}