using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net.Http;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using Microsoft.ApplicationInsights;
using Microsoft.ApplicationInsights.DataContracts;
using Microsoft.Azure.OperationalInsights;
using Microsoft.Extensions.Options;
using Microsoft.Rest.Azure.Authentication;
using Newtonsoft.Json;
using NTK2019.Web.Interfaces;
using NTK2019.Web.Models;
using NTK2019.Web.Options;
using MediaTypeHeaderValue = System.Net.Http.Headers.MediaTypeHeaderValue;

namespace NTK2019.Web.Services
{
    public class LogAnalyticsService : ILogAnalyticsService
    {
        private readonly TelemetryClient telemetryClient = new TelemetryClient();
        private readonly LogAnalyticsOptions analyticsOptions;

        public LogAnalyticsService(IOptions<LogAnalyticsOptions> options)
        {
            analyticsOptions = options.Value;
        }

        public async Task<List<LogAnalyticsResult>> ExecuteQueryAsync(string query)
        {
            var adSettings = new ActiveDirectoryServiceSettings
            {
                AuthenticationEndpoint = new Uri(analyticsOptions.AadAuthPoint),
                TokenAudience = new Uri(analyticsOptions.AadTokenAudience),
                ValidateAuthority = true
            };

            var creds = await ApplicationTokenProvider.LoginSilentAsync(analyticsOptions.AadDomain,
                analyticsOptions.ClientId,
                analyticsOptions.ClientSecret, adSettings);
            telemetryClient.TrackEvent("ExecuteQueryAsync", new Dictionary<string, string>
            {
                {
                    "Status", "user silently logged in"
                }
            });
            var client = new OperationalInsightsDataClient(creds) {WorkspaceId = analyticsOptions.WorkspaceId};

            var results = client.Query(query);

            telemetryClient.TrackEvent("ExecuteQueryAsync", new Dictionary<string, string>
            {
                {
                    "Status", $"Pridobili smo podatke! {results.Results.Count()} rezultatov!"
                }
            });

            var list = new List<LogAnalyticsResult>();
            foreach (var resultsResult in results.Results)
            {
                list.Add(new LogAnalyticsResult {Key = resultsResult.Keys, Value = resultsResult.Values});
            }

            telemetryClient.Flush();

            return list;
        }

        public async Task<bool> AddCustomLogAsync(string customLogName, string message)
        {
            try
            {
                var watch = new Stopwatch();
                watch.Start();
                //we could extend it
                var list = new List<CustomLogViewModel>
                {
                    new CustomLogViewModel {Message = message, CollectionName = customLogName}
                };

                var json = JsonConvert.SerializeObject(list);

                var datestring = DateTime.UtcNow.ToString("r");
                var jsonBytes = Encoding.UTF8.GetBytes(json);
                string stringToHash = $"POST\n{jsonBytes.Length}\napplication/json\nx-ms-date:{datestring}\n/api/logs";
                string hashedString = BuildSignature(stringToHash, analyticsOptions.SharedKey);
                string signature = $"SharedKey {analyticsOptions.WorkspaceId}:{hashedString}";

                string url =
                    $"https://{analyticsOptions.WorkspaceId}.ods.opinsights.azure.com/api/logs?api-version=2016-04-01";

                var client = new HttpClient();
                client.DefaultRequestHeaders.Add("Accept", "application/json");
                client.DefaultRequestHeaders.Add("Log-Type", customLogName);
                client.DefaultRequestHeaders.Add("Authorization", signature);
                client.DefaultRequestHeaders.Add("x-ms-date", datestring);
                client.DefaultRequestHeaders.Add("time-generated-field", string.Empty);

                HttpContent httpContent = new StringContent(json, Encoding.UTF8);
                httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");
                var response = await client.PostAsync(new Uri(url), httpContent);

                var responseContent = response.Content;
                watch.Stop();

                var hasSuccedeed = response.IsSuccessStatusCode;

                telemetryClient.TrackRequest(new RequestTelemetry("ods.opinsights.azure.com/api/logs",
                    DateTimeOffset.Now, watch.Elapsed, response.StatusCode.ToString(),
                    hasSuccedeed));

                if (hasSuccedeed)
                {
                    string result = await responseContent.ReadAsStringAsync();
                    telemetryClient.TrackEvent("AddCustomLogAsync", new Dictionary<string, string>
                    {
                        {
                            "Result", result
                        },
                        {
                            "ElapsedTime", $"{watch.Elapsed.Milliseconds} ms"
                        }
                    });
                }

                return hasSuccedeed;
            }
            catch (Exception e)
            {
                telemetryClient.TrackException(e);
                return false;
            }
            finally
            {
                telemetryClient.Flush();
            }
        }

        // Build the API signature
        private string BuildSignature(string message, string secret)
        {
            var encoding = new ASCIIEncoding();
            byte[] keyByte = Convert.FromBase64String(secret);

            byte[] messageBytes = encoding.GetBytes(message);
            using (var hmacsha256 = new HMACSHA256(keyByte))
            {
                byte[] hash = hmacsha256.ComputeHash(messageBytes);
                return Convert.ToBase64String(hash);
            }
        }
    }
}