using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.ApplicationInsights;
using Microsoft.Azure.Management.Fluent;
using Microsoft.Azure.Management.ResourceManager.Fluent;
using Microsoft.Azure.Management.ResourceManager.Fluent.Core;
using Microsoft.Extensions.Options;
using NTK2019.Web.Interfaces;
using NTK2019.Web.Models;
using NTK2019.Web.Options;

namespace NTK2019.Web.Services
{
    public class MonitorService : IMonitorService
    {
        private readonly TelemetryClient telemetryClient = new TelemetryClient();
        private readonly LogAnalyticsOptions analyticsOptions;
        private readonly StorageOptions storageOptions;

        public MonitorService(IOptions<LogAnalyticsOptions> options, IOptions<StorageOptions> storageOptionsValue)
        {
            analyticsOptions = options.Value;
            storageOptions = storageOptionsValue.Value;
        }

        public async Task<List<MonitorViewModel>> GetMetricsForStorageAsync()
        {
            var credentials = SdkContext.AzureCredentialsFactory.FromServicePrincipal(analyticsOptions.ClientId, 
                analyticsOptions.ClientSecret, 
                analyticsOptions.TenantId, 
                AzureEnvironment.AzureGlobalCloud);
 
            var azure = Azure
                .Configure()
                .WithLogLevel(HttpLoggingDelegatingHandler.Level.Basic)
                .Authenticate(credentials).WithSubscription(analyticsOptions.SubscriptionId);
            
            string storageId = "";
            foreach (var storageAccount in azure.StorageAccounts.List())
            {
                if (storageAccount.Name == storageOptions.Name)
                    storageId = storageAccount.Id;
            }
            
            if (string.IsNullOrEmpty(storageId)) throw new MissingFieldException("Storage id not found!");
            
            var recordDateTime = DateTime.Now;
            
            var list = new List<MonitorViewModel>();
            foreach (var metricDefinition in await azure.MetricDefinitions.ListByResourceAsync(storageId))
            {
                if (metricDefinition.Name.LocalizedValue.Equals("transactions", StringComparison.OrdinalIgnoreCase))
                {
                    // get metric records
                    var metricCollection = metricDefinition.DefineQuery()
                        .StartingFrom(recordDateTime.AddDays(-7))
                        .EndsBefore(recordDateTime)
                        .WithAggregation("Average")
                        .WithInterval(TimeSpan.FromMinutes(5))
                        .WithOdataFilter("apiName eq 'PutBlob' and responseType eq 'Success'")
                        .Execute();
                    
                    foreach (var metric in metricCollection.Metrics)
                    {
                        list.Add(new MonitorViewModel{Type = "metric", Value = metric.Name.LocalizedValue});
                    }
                    break;
                }
            }
            
            var logs = await azure.ActivityLogs.DefineQuery()
                .StartingFrom(recordDateTime.AddDays(-7))
                .EndsBefore(recordDateTime)
                .WithAllPropertiesInResponse()
                .FilterByResource(storageId)
                .ExecuteAsync();
            
            foreach (var eventData in logs)
            {
                list.Add(new MonitorViewModel{Type = "activitylog", Value = eventData.EventName?.LocalizedValue + " " + eventData.OperationName?.LocalizedValue});
            }

            return list;
        }
    }
}